@@ -69,6 +69,9 @@ GtkWidget *gui_blocklist_length, *gui_blocklist_effectpages;
 GtkWidget *gui_instrument_window, *gui_instrument_name, *gui_instrument_volume;
 GtkWidget *gui_instrument_transpose, *gui_instrument_hold, *gui_instrument_midichannel;
 
+/* Track volumes window GTK widgets */
+GtkWidget *gui_trackvolumes_window, *gui_trackvolumes_hbox;
+
 void delete_event(GtkWidget *w, GdkEvent *e, gpointer d) {
   gtk_main_quit();
 }
@@ -212,7 +215,7 @@ void gui_create() {
   w4=gtk_menu_item_new_with_label("Track Volumes... AltGr+V");
   gtk_menu_append(GTK_MENU(w3), w4);
   gtk_signal_connect(GTK_OBJECT(w4), "activate",
-		     GTK_SIGNAL_FUNC(gui_tempo_show), NULL);
+		     GTK_SIGNAL_FUNC(gui_trackvolumes_show), NULL);
 
   w2=gtk_menu_item_new_with_label("Block");
   gtk_menu_bar_append(GTK_MENU_BAR(w1), w2);
@@ -287,14 +290,14 @@ void gui_create() {
   w2=gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(w1), w2);
   /* Horizontal box for instrument number, name and type */
-  w1=gtk_hbox_new(FALSE, 4);
+  w1=gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(w2), w1, FALSE, FALSE, 4);
   gui_main_instrument=gtk_label_new("");
-  gtk_box_pack_start(GTK_BOX(w1), gui_main_instrument, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(w1), gui_main_instrument, FALSE, FALSE, 4);
   gui_main_instrument_name=gtk_entry_new();
   gtk_box_pack_start(GTK_BOX(w1), gui_main_instrument_name, TRUE, TRUE, 0);
   w2=gtk_button_new_with_label("Properties");
-  gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 4);
+  gtk_box_pack_start(GTK_BOX(w1), w2, FALSE, FALSE, 4);
   gtk_signal_connect(GTK_OBJECT(w2), "clicked",
 		     GTK_SIGNAL_FUNC(gui_instrument_show), NULL);
   /* Horizontal box for edit, space, chord and octave selector */
@@ -455,6 +458,30 @@ void gui_create() {
 		     GTK_SIGNAL_FUNC(gui_tempo_hide), NULL);
   gtk_box_pack_start(GTK_BOX(w1), w3, FALSE, FALSE, 0);
 
+  /* Tempo window */
+  gui_trackvolumes_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_container_set_border_width(GTK_CONTAINER(gui_trackvolumes_window), 4);
+  gtk_window_set_title(GTK_WINDOW(gui_trackvolumes_window), "Track Volumes");
+  gtk_widget_add_events(GTK_WIDGET(gui_trackvolumes_window), GDK_KEY_RELEASE_MASK);
+  gtk_signal_connect(GTK_OBJECT(gui_trackvolumes_window), "key_press_event",
+		     GTK_SIGNAL_FUNC(keyboard_event), (gpointer)1);
+  gtk_signal_connect(GTK_OBJECT(gui_trackvolumes_window), "key_release_event",
+		     GTK_SIGNAL_FUNC(keyboard_event), (gpointer)0);
+
+  w1=gtk_vbox_new(FALSE, 4);
+  gtk_container_add(GTK_CONTAINER(gui_trackvolumes_window), w1);
+
+  w2=gtk_frame_new(NULL);
+  gtk_frame_set_shadow_type(GTK_FRAME(w2), GTK_SHADOW_ETCHED_IN);
+  gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 0);
+  gui_trackvolumes_hbox=gtk_hbox_new(FALSE, 4);
+  gtk_container_add(GTK_CONTAINER(w2), gui_trackvolumes_hbox);
+
+  w3=gtk_button_new_with_label("Close");
+  gtk_signal_connect(GTK_OBJECT(w3), "clicked",
+		     GTK_SIGNAL_FUNC(gui_trackvolumes_hide), NULL);
+  gtk_box_pack_start(GTK_BOX(w1), w3, FALSE, FALSE, 0);
+
   /* Section list window */
   gui_sectionlist_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_container_set_border_width(GTK_CONTAINER(gui_sectionlist_window), 4);
@@ -724,55 +751,65 @@ void gui_create() {
   gtk_signal_connect(GTK_OBJECT(gui_instrument_window), "key_release_event",
 		     GTK_SIGNAL_FUNC(keyboard_event), (gpointer)0);
 
-  w1=gtk_vbox_new(FALSE, 0);
+  w1=gtk_vbox_new(FALSE, 4);
   gtk_container_add(GTK_CONTAINER(gui_instrument_window), w1);
-  w2=gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 0);
-  w3=gtk_label_new("Name");
-  gtk_box_pack_start(GTK_BOX(w2), w3, FALSE, FALSE, 4);
+  w3=gtk_frame_new(NULL);
+  gtk_frame_set_shadow_type(GTK_FRAME(w3), GTK_SHADOW_ETCHED_IN);
+  gtk_box_pack_start(GTK_BOX(w1), w3, TRUE, TRUE, 0);
+  w2=gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(w3), w2);
+  w3=gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(w2), w3, TRUE, TRUE, 4);
+  w4=gtk_label_new("Name");
+  gtk_box_pack_start(GTK_BOX(w3), w4, FALSE, FALSE, 4);
   gui_instrument_name=gtk_entry_new();
-  gtk_box_pack_start(GTK_BOX(w2), gui_instrument_name, TRUE, TRUE, 0);
-  w2=gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 0);
-  w3=gtk_label_new("Volume");
-  gtk_box_pack_start(GTK_BOX(w2), w3, FALSE, FALSE, 4);
+  gtk_box_pack_start(GTK_BOX(w3), gui_instrument_name, TRUE, TRUE, 4);
+  w3=gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(w2), w3, TRUE, TRUE, 0);
+  w4=gtk_label_new("Volume");
+  gtk_box_pack_start(GTK_BOX(w3), w4, FALSE, FALSE, 4);
   gui_instrument_volume=(GtkWidget *)gtk_adjustment_new(127, 0, 127, 1, 1, 0);
   gtk_signal_connect(GTK_OBJECT(gui_instrument_volume), "value_changed",
 		     GTK_SIGNAL_FUNC(gui_instrument_volume_changed), NULL);
-  w3=gtk_hscale_new(GTK_ADJUSTMENT(gui_instrument_volume));
-  gtk_scale_set_digits(GTK_SCALE(w3), 0);
-  gtk_box_pack_start(GTK_BOX(w2), w3, TRUE, TRUE, 4);
-  w2=gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 0);
-  w3=gtk_label_new("Transpose");
-  gtk_box_pack_start(GTK_BOX(w2), w3, FALSE, FALSE, 4);
+  w4=gtk_hscale_new(GTK_ADJUSTMENT(gui_instrument_volume));
+  gtk_scale_set_digits(GTK_SCALE(w4), 0);
+  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
+  w3=gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(w2), w3, TRUE, TRUE, 0);
+  w4=gtk_label_new("Transpose");
+  gtk_box_pack_start(GTK_BOX(w3), w4, FALSE, FALSE, 4);
   gui_instrument_transpose=(GtkWidget *)gtk_adjustment_new(0, -12, 12, 1, 1, 0);
   gtk_signal_connect(GTK_OBJECT(gui_instrument_transpose), "value_changed",
 		     GTK_SIGNAL_FUNC(gui_instrument_transpose_changed), NULL);
-  w3=gtk_hscale_new(GTK_ADJUSTMENT(gui_instrument_transpose));
-  gtk_scale_set_digits(GTK_SCALE(w3), 0);
-  gtk_box_pack_start(GTK_BOX(w2), w3, TRUE, TRUE, 4);
-  w2=gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 0);
-  w3=gtk_label_new("Hold");
-  gtk_box_pack_start(GTK_BOX(w2), w3, FALSE, FALSE, 4);
+  w4=gtk_hscale_new(GTK_ADJUSTMENT(gui_instrument_transpose));
+  gtk_scale_set_digits(GTK_SCALE(w4), 0);
+  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
+  w3=gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(w2), w3, TRUE, TRUE, 0);
+  w4=gtk_label_new("Hold");
+  gtk_box_pack_start(GTK_BOX(w3), w4, FALSE, FALSE, 4);
   gui_instrument_hold=(GtkWidget *)gtk_adjustment_new(0, 0, 255, 1, 1, 0);
   gtk_signal_connect(GTK_OBJECT(gui_instrument_hold), "value_changed",
 		     GTK_SIGNAL_FUNC(gui_instrument_hold_changed), NULL);
-  w3=gtk_hscale_new(GTK_ADJUSTMENT(gui_instrument_hold));
-  gtk_scale_set_digits(GTK_SCALE(w3), 0);
+  w4=gtk_hscale_new(GTK_ADJUSTMENT(gui_instrument_hold));
+  gtk_scale_set_digits(GTK_SCALE(w4), 0);
+  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
+  w3=gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(w2), w3, TRUE, TRUE, 4);
-  w2=gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 0);
-  w3=gtk_label_new("MIDI channel");
-  gtk_box_pack_start(GTK_BOX(w2), w3, FALSE, FALSE, 4);
+  w4=gtk_label_new("MIDI channel");
+  gtk_box_pack_start(GTK_BOX(w3), w4, FALSE, FALSE, 4);
   gui_instrument_midichannel=(GtkWidget *)gtk_adjustment_new(1, 1, 16, 1, 1, 0);
   gtk_signal_connect(GTK_OBJECT(gui_instrument_midichannel), "value_changed",
 		     GTK_SIGNAL_FUNC(gui_instrument_midichannel_changed),
 		     NULL);
-  w3=gtk_hscale_new(GTK_ADJUSTMENT(gui_instrument_midichannel));
-  gtk_scale_set_digits(GTK_SCALE(w3), 0);
-  gtk_box_pack_start(GTK_BOX(w2), w3, TRUE, TRUE, 4);
+  w4=gtk_hscale_new(GTK_ADJUSTMENT(gui_instrument_midichannel));
+  gtk_scale_set_digits(GTK_SCALE(w4), 0);
+  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
+
+  w3=gtk_button_new_with_label("Close");
+  gtk_signal_connect(GTK_OBJECT(w3), "clicked",
+		     GTK_SIGNAL_FUNC(gui_instrument_hide), NULL);
+  gtk_box_pack_start(GTK_BOX(w1), w3, FALSE, FALSE, 0);
 
   /* File selection windows */
   gui_file_open=gtk_file_selection_new("Open file...");
@@ -826,6 +863,54 @@ void gui_tempo_hide() {
   gtk_widget_hide(gui_tempo_window);
 }
 
+void gui_trackvolumes_refresh() {
+  int i;
+  GList *children;
+  GtkWidget *w1, *w2, *w3, *w4;
+  char track[4];
+
+  /* Destroy all children */
+  children=gtk_container_children(GTK_CONTAINER(gui_trackvolumes_hbox));
+  while(children!=NULL) {
+    gtk_widget_destroy(GTK_WIDGET(children->data));
+    children=children->next;
+  }
+
+  for(i=0; i<editor.song->maxtracks; i++) {
+    w1=gtk_vbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(gui_trackvolumes_hbox), w1, TRUE, TRUE, 4);
+
+    /* Volume sliders */
+    w2=(GtkWidget *)gtk_adjustment_new(editor.song->trackvolumes[i], 0, 127,
+				       1, 1, 0);
+    gtk_signal_connect(GTK_OBJECT(w2), "value_changed",
+		       GTK_SIGNAL_FUNC(gui_trackvolumes_changed),
+		       (gpointer)&editor.song->trackvolumes[i]);
+    w3=gtk_vscale_new(GTK_ADJUSTMENT(w2));
+    gtk_scale_set_digits(GTK_SCALE(w3), 0);
+    gtk_box_pack_start(GTK_BOX(w1), w3, TRUE, TRUE, 4);
+
+    /* Mute buttons */
+    snprintf(track, 4, "%d", i+1);
+    w4=gtk_check_button_new_with_label(track);
+    gtk_signal_connect(GTK_OBJECT(w4), "toggled",
+		       GTK_SIGNAL_FUNC(gui_trackvolumes_toggled),
+		       (gpointer)&editor.song->trackvolumes[i]);
+    gtk_box_pack_start(GTK_BOX(w1), w4, FALSE, FALSE, 4);
+  }
+
+  gtk_widget_show_all(gui_trackvolumes_hbox);
+}
+
+void gui_trackvolumes_show() {
+  gtk_window_set_default_size(GTK_WINDOW(gui_trackvolumes_window), 200, 200);
+  gtk_widget_show_all(gui_trackvolumes_window);
+}
+
+void gui_trackvolumes_hide() {
+  gtk_widget_hide(gui_trackvolumes_window);
+}
+
 void gui_info_refresh() {
   char section[9+8], playseq[10+8], block[7+9], title[76];
 
@@ -972,17 +1057,17 @@ void gui_playseq_activate_name(GtkWidget *widget, GtkWidget *entry) {
 }
 
 void gui_sectionlist_insert() {
-  song_insertsection(editor.song, editor.section);
+  song_insert_section(editor.song, editor.section);
   gui_sectionlist_refresh();
 }
 
 void gui_sectionlist_append() {
-  song_insertsection(editor.song, editor.song->numsections);
+  song_insert_section(editor.song, editor.song->numsections);
   gui_sectionlist_refresh();
 }
 
 void gui_sectionlist_delete() {
-  song_deletesection(editor.song, editor.section);
+  song_delete_section(editor.song, editor.section);
   if(editor.section>=editor.song->numsections)
     editor.section=editor.song->numsections-1;
 
@@ -1093,7 +1178,7 @@ void gui_playseq_delete() {
 
 void gui_playseq_insertnew() {
   if(editor.song!=NULL) {
-    song_insertplayseq(editor.song, editor.playseq);
+    song_insert_playseq(editor.song, editor.playseq);
     gui_info_refresh();
     gui_playseq_refresh();
     gui_sectionlist_refresh();
@@ -1102,7 +1187,7 @@ void gui_playseq_insertnew() {
 
 void gui_playseq_appendnew() {
   if(editor.song!=NULL) {
-    song_insertplayseq(editor.song, editor.song->numplayseqs);
+    song_insert_playseq(editor.song, editor.song->numplayseqs);
     gui_info_refresh();
     gui_playseq_refresh();
     gui_sectionlist_refresh();
@@ -1111,7 +1196,7 @@ void gui_playseq_appendnew() {
 
 void gui_playseq_deletecurrent() {
   if(editor.song!=NULL) {
-    song_deleteplayseq(editor.song, editor.playseq);
+    song_delete_playseq(editor.song, editor.playseq);
     gui_info_refresh();
     gui_playseq_refresh();
     gui_sectionlist_refresh();
@@ -1218,7 +1303,7 @@ void gui_blocklist_activate_name(GtkWidget *widget, GtkWidget *entry) {
 
 void gui_blocklist_insert() {
   if(editor.song!=NULL) {
-    song_insertblock(editor.song, editor.block);
+    song_insert_block(editor.song, editor.block);
     gui_blocklist_refresh();
     gui_playseq_refresh();
   }
@@ -1226,7 +1311,7 @@ void gui_blocklist_insert() {
 
 void gui_blocklist_append() {
   if(editor.song!=NULL) {
-    song_insertblock(editor.song, editor.song->numblocks);
+    song_insert_block(editor.song, editor.song->numblocks);
     gui_blocklist_refresh();
     gui_playseq_refresh();
   }
@@ -1234,7 +1319,7 @@ void gui_blocklist_append() {
 
 void gui_blocklist_delete() {
   if(editor.song!=NULL) {
-    song_deleteblock(editor.song, editor.block);
+    song_delete_block(editor.song, editor.block);
     gui_blocklist_refresh();
     gui_playseq_refresh();
   }
@@ -1248,8 +1333,10 @@ void gui_blocklist_tracks_changed(GtkAdjustment *adjustment,
     tracker_set_num_channels((Tracker *)gui_main_tracker,
 			     (int)adjustment->value);
     tracker_redraw((Tracker *)gui_main_tracker);
+    if(song_check_maxtracks(editor.song))
+      gui_trackvolumes_refresh();
   }
- }
+}
 
 void gui_blocklist_length_changed(GtkAdjustment *adjustment,
 				  gpointer user_data) {
@@ -1300,11 +1387,25 @@ void gui_main_changed(GtkAdjustment *adjustment, gpointer user_data) {
 }
 
 void gui_tempo_changed(GtkAdjustment *adjustment, gpointer user_data) {
-  song_settempo(editor.song, (unsigned char)adjustment->value);
+  song_set_tempo(editor.song, (unsigned char)adjustment->value);
   gui_info_refresh();
 }
 
 void gui_tpl_changed(GtkAdjustment *adjustment, gpointer user_data) {
-  song_settpl(editor.song, (unsigned char)adjustment->value);
+  song_set_tpl(editor.song, (unsigned char)adjustment->value);
   gui_info_refresh();
 }
+
+void gui_trackvolumes_changed(GtkAdjustment *adjustment, gpointer user_data) {
+  unsigned char *data=(unsigned char *)user_data;
+  *data=((*data)&128)|((unsigned char)adjustment->value);
+}
+
+void gui_trackvolumes_toggled(GtkToggleButton *togglebutton,
+			      gpointer user_data) {
+  unsigned char *data=(unsigned char *)user_data;
+  if(gtk_toggle_button_get_active(togglebutton))
+    *data=((*data)&127)|128;
+  else
+    *data=(*data)&127;
+}
