@@ -30,6 +30,7 @@ extern struct editor editor;
 /* Allocates a new song structure and initializes it to default values */
 struct song *song_alloc(void) {
   struct song *song;
+  int i;
 
   song=(struct song *)calloc(1, sizeof(struct song));
   song->name=(char *)strdup("Untitled");
@@ -48,6 +49,9 @@ struct song *song_alloc(void) {
   song->instruments[0]=instrument_alloc();
   song->maxtracks=4;
   song->trackvolumes=(char *)calloc(4, sizeof(char));
+  song->mastervolume=127;
+  for(i=0; i<song->maxtracks; i++)
+    song->trackvolumes[i]=127;
 
   return(song);
 }
@@ -97,7 +101,7 @@ void song_free(struct song *song) {
 }
 
 /* Inserts a new block in the block array in the given position */
-void song_insertblock(struct song *song, int pos) {
+void song_insert_block(struct song *song, int pos) {
   int i;
 
   /* Check block existence */
@@ -130,7 +134,7 @@ void song_insertblock(struct song *song, int pos) {
 }
 
 /* Deletes a block from the given position of the block array */
-void song_deleteblock(struct song *song, int pos) {
+void song_delete_block(struct song *song, int pos) {
   int i;
 
   /* Check block existence */
@@ -164,7 +168,7 @@ void song_deleteblock(struct song *song, int pos) {
 }
 
 /* Inserts a new playseq in the playseq array in the given position */
-void song_insertplayseq(struct song *song, int pos) {
+void song_insert_playseq(struct song *song, int pos) {
   int i;
 
   /* Check playseq existence */
@@ -192,7 +196,7 @@ void song_insertplayseq(struct song *song, int pos) {
 }
 
 /* Deletes a playseq from the given position of the playseq array */
-void song_deleteplayseq(struct song *song, int pos) {
+void song_delete_playseq(struct song *song, int pos) {
   int i;
   
   /* Check block existence */
@@ -220,7 +224,7 @@ void song_deleteplayseq(struct song *song, int pos) {
 
 
 /* Inserts a new section in the section array in the given position */
-void song_insertsection(struct song *song, int pos) {
+void song_insert_section(struct song *song, int pos) {
   int i;
 
   /* Check block existence */
@@ -239,7 +243,7 @@ void song_insertsection(struct song *song, int pos) {
 }
 
 /* Deletes a section from the given position of the section array */
-void song_deletesection(struct song *song, int pos) {
+void song_delete_section(struct song *song, int pos) {
   int i;
   
   /* Check block existence */
@@ -264,23 +268,47 @@ void song_deletesection(struct song *song, int pos) {
 }
 
 /* Sets a section in the given position to point somewhere */
-void song_setsection(struct song *song, int pos, int playseq) {
+void song_set_section(struct song *song, int pos, int playseq) {
   if(playseq>=0 && playseq<song->numplayseqs)
     song->sections[pos]=playseq;
 }
 
 /* Sets the number of ticks per line for a song */
-void song_settpl(struct song *song, unsigned char ticksperline) {
+void song_set_tpl(struct song *song, unsigned char ticksperline) {
   song->ticksperline=ticksperline;
 }
 
 /* Sets the tempo of a song */
-void song_settempo(struct song *song, unsigned char tempo) {
+void song_set_tempo(struct song *song, unsigned char tempo) {
   song->tempo=tempo;
 }
 
+/* If the maximum number of tracks has changed recreate the track volumes */
+int song_check_maxtracks(struct song *song) {
+  int i, oldmax=song->maxtracks, max=0;
+
+  /* Check the maximum number of tracks; */
+  for(i=0; i<song->numblocks; i++)
+    if(song->blocks[i]->tracks>max)
+      max=song->blocks[i]->tracks;
+  
+  if(oldmax!=max) {
+    /* Reallocate volume array */
+    song->trackvolumes=(unsigned char *)realloc(song->trackvolumes,
+						max*sizeof(unsigned char));
+
+    /* Set new tracks to volume 127 */
+    for(i=song->maxtracks; i<max; i++)
+      song->trackvolumes[i]=127;
+
+    song->maxtracks=max;
+    return 1;
+  } else
+    return 0;
+}
+
 /* Converts an MMD2 module to a song */
-struct song *song_convertMMD2(struct MMD2 *mmd) {
+struct song *song_convert_MMD2(struct MMD2 *mmd) {
   struct song *song;
   int i, j, k, l;
   unsigned char effectconversion[256];
@@ -429,6 +457,16 @@ struct song *song_convertMMD2(struct MMD2 *mmd) {
     }
     song->maxtracks=mmd->song->numtracks;
     song->trackvolumes=(char *)calloc(song->maxtracks, sizeof(char));
+    for(i=0; i<song->maxtracks; i++) {
+      if(mmd->song->trackvols[i]<64)
+	song->trackvolumes[i]=mmd->song->trackvols[i]*2;
+      else
+	song->trackvolumes[i]=127;
+    }
+    if(mmd->song->mastervol<64)
+      song->mastervolume=mmd->song->mastervol*2;
+    else
+      song->mastervolume=127;
   } else {
     song->tempo=DEFAULT_TEMPO;
     song->ticksperline=6;
@@ -462,7 +500,7 @@ struct song *song_load(char *filename) {
     struct MMD2 *mmd;
     struct song *song;
     mmd=MMD2_load(filename);
-    song=song_convertMMD2(mmd);
+    song=song_convert_MMD2(mmd);
     MMD2_free(mmd);
     if(song!=NULL)
       return song;
@@ -490,6 +528,9 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
     prop=xmlGetProp(cur, "ticksperline");
     if(prop!=NULL)
       song->ticksperline=atoi(prop);
+    prop=xmlGetProp(cur, "mastervolume");
+    if(prop!=NULL)
+      song->mastervolume=atoi(prop);
 
     cur=cur->xmlChildrenNode;
     while(cur!=NULL) {
@@ -503,7 +544,8 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 	    struct block *block=block_parse(doc, ns, temp);
 	    if(block!=NULL) {
 	      song->numblocks++;
-	      song->blocks=realloc(song->blocks, song->numblocks*sizeof(struct block *));
+	      song->blocks=realloc(song->blocks,
+				   song->numblocks*sizeof(struct block *));
 	      song->blocks[song->numblocks-1]=block;
 	    }
 	  }
@@ -517,7 +559,7 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 	temp=cur->xmlChildrenNode;
 	while(temp!=NULL) {
 	  if(!(xmlIsBlankNode(temp))) {
-	    prop=xmlGetProp(cur, "playseq");
+	    prop=xmlGetProp(temp, "playseq");
 	    if(prop!=NULL) {
 	      song->numsections++;
 	      song->sections=realloc(song->sections, song->numsections*sizeof(unsigned short));
@@ -559,6 +601,24 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 	    }
 	  }
 	  
+	  temp=temp->next;
+	}
+      } else if((!xmlStrcmp(cur->name, "trackvolumes")) && (cur->ns==ns)) {
+	xmlNodePtr temp;
+
+	/* Parse and add all track volume elements */
+	temp=cur->xmlChildrenNode;
+	while(temp!=NULL) {
+	  if(!(xmlIsBlankNode(temp))) {
+	    prop=xmlGetProp(temp, "volume");
+	    if(prop!=NULL) {
+	      song->maxtracks++;
+	      song->trackvolumes=realloc(song->trackvolumes,
+				     song->maxtracks*sizeof(unsigned char));
+	      song->trackvolumes[song->maxtracks-1]=atoi(prop);
+	    }
+	  }
+	  
 	  temp=temp->next;
 	}
       }
@@ -584,6 +644,8 @@ void song_save(struct song *song, char *filename) {
   xmlSetProp(doc->xmlRootNode, "tempo", c);
   snprintf(c, 10, "%d", song->ticksperline);
   xmlSetProp(doc->xmlRootNode, "ticksperline", c);
+  snprintf(c, 10, "%d", song->mastervolume);
+  xmlSetProp(doc->xmlRootNode, "mastervolume", c);
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
@@ -603,7 +665,9 @@ void song_save(struct song *song, char *filename) {
   for(i=0; i<song->numsections; i++) {
     subnode=xmlNewChild(node, NULL, "section", NULL);
     snprintf(c, 10, "%d", song->sections[i]);
-    xmlSetProp(doc->xmlRootNode, "playseq", c);
+    xmlSetProp(subnode, "playseq", c);
+    lb=xmlNewText("\n");
+    xmlAddChild(node, lb);
   }
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
@@ -626,6 +690,20 @@ void song_save(struct song *song, char *filename) {
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
+  node=xmlNewChild(doc->xmlRootNode, NULL, "trackvolumes", NULL);
+  lb=xmlNewText("\n");
+  xmlAddChild(node, lb);
+  /* Add all track volumes */
+  for(i=0; i<song->maxtracks; i++) {
+    subnode=xmlNewChild(node, NULL, "track", NULL);
+    snprintf(c, 10, "%d", song->trackvolumes[i]);
+    xmlSetProp(subnode, "volume", c);
+    lb=xmlNewText("\n");
+    xmlAddChild(node, lb);
+  }
+  lb=xmlNewText("\n\n");
+  xmlAddChild(doc->xmlRootNode, lb);
+
   /* Use stdout if no filename is specified */
   if(filename==NULL)
     filename="-";
